@@ -103,6 +103,7 @@ class App(Gtk.Application, TimerManager):
 		self.send_limit = -1			# -//-
 		self.ur_question_shown = False	# Used to prevent showing 'Do you wan't usage reporting'
 										# question more than once until ST-GTK is restarted.
+		self.home_dir_override = None	# If set by '--home'
 		self.wizard = None
 		self.widgets = {}
 		self.error_boxes = []
@@ -120,9 +121,6 @@ class App(Gtk.Application, TimerManager):
 		self.setup_widgets()
 		self.setup_actions()
 		self.setup_statusicon()
-		if self.wizard == None:
-			if self.setup_connection():
-				self.daemon.reconnect()
 	
 	def do_local_options(self, trash, lo):
 		self.parse_local_options(lo.contains)
@@ -149,6 +147,8 @@ class App(Gtk.Application, TimerManager):
 			if cl.get_options_dict().contains("quit"):
 				self.cb_exit()
 				return 0
+			if cl.get_options_dict().contains("home"):
+				self.home_dir_override = cl.get_options_dict().lookup_value("home").get_string()
 			if not StDownloader is None:
 				if cl.get_options_dict().contains("force-update"):
 					self.force_update_version = \
@@ -187,6 +187,11 @@ class App(Gtk.Application, TimerManager):
 							return True
 				return False
 			self.parse_local_options(is_option)
+		
+		if self.daemon == None:
+			if self.wizard == None:
+				if self.setup_connection():
+					self.daemon.reconnect()
 		self.activate()
 		return 0
 	
@@ -232,6 +237,8 @@ class App(Gtk.Application, TimerManager):
 		aso("debug",	b"d", "Be more verbose (debug mode)")
 		aso("wizard",	b"1", "Run 'first start wizard' and exit")
 		aso("about",	b"a", "Display about dialog and exit")
+		aso("home", 0, "Overrides default syncthing configuration directory",
+				GLib.OptionArg.STRING)
 		aso("add-repo", 0,    "Opens 'add repository' dialog with specified path prefilled",
 				GLib.OptionArg.STRING)
 		aso("remove-repo", 0, "If there is repository assigned with specified path, opens 'remove repository' dialog",
@@ -315,7 +322,10 @@ class App(Gtk.Application, TimerManager):
 	def setup_connection(self):
 		# Create Daemon instance (loads and parses config)
 		try:
-			self.daemon = Daemon()
+			if self.home_dir_override:
+				self.daemon = Daemon(os.path.join(self.home_dir_override, "config.xml"))
+			else:
+				self.daemon = Daemon(self.home_dir_override)
 		except InvalidConfigurationException, e:
 			# Syncthing is not configured, most likely never launched.
 			# Run wizard.
@@ -413,6 +423,8 @@ class App(Gtk.Application, TimerManager):
 		cmdline = [self.config["syncthing_binary"], "-no-browser"]
 		vars, preargs, args = parse_config_arguments(self.config["syncthing_arguments"])
 		cmdline = preargs + cmdline + args
+		if self.home_dir_override:
+			cmdline += [ "-home" , self.home_dir_override ]
 		
 		self.process = DaemonProcess(cmdline, self.config["daemon_priority"], self.config["max_cpus"], env=vars)
 		self.process.connect('failed', self.cb_daemon_startup_failed)
@@ -609,7 +621,6 @@ class App(Gtk.Application, TimerManager):
 				)
 			self.show_info_box(r)
 			self.cb_infobar_close(r)
-			
 	
 	def cb_syncthing_disconnected(self, daemon, reason, message):
 		# if reason == Daemon.UNEXPECTED
@@ -680,6 +691,12 @@ class App(Gtk.Application, TimerManager):
 						_("Connection to daemon failed. Check your configuration and try again."),
 						_("Error message:"), str(message)
 						)
+				if "Not found" in str(message):
+					# Special case that has usual explanation
+					message = "%s\n%s" % (
+						message,
+						_("Possible cause: Is there another web server running on Syncthing port?")
+					)
 			d = Gtk.MessageDialog(
 					self["window"],
 					Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
@@ -704,6 +721,7 @@ class App(Gtk.Application, TimerManager):
 				swin = Gtk.ScrolledWindow()
 				swin.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
 				swin.add_with_viewport(tview)
+				swin.set_size_request(300, 400)
 				ex.add(swin)
 				d.get_message_area().pack_end(ex, True, True, 1)
 				ex.show_all()
@@ -752,7 +770,9 @@ class App(Gtk.Application, TimerManager):
 			self.watcher.start()
 	
 	def cb_syncthing_error(self, daemon, message):
-		""" Daemon argument is not used """
+		""" Handles errors reported by syncthing daemon """
+		# Daemon argument is not used
+		RE_IP_PORT = re.compile(r"([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+):([0-9]+)")
 		if "remote device speaks an older version of the protocol" in message:
 			# This one needs special treatement because remote port changes
 			# every time when this is reported.
@@ -773,6 +793,11 @@ class App(Gtk.Application, TimerManager):
 			
 			message = _("Connecting to <b>%s</b> failed; the remote device speaks an older version of the protocol (%s) not compatible with this version") % (
 					display_id, version)
+		while RE_IP_PORT.search(message):
+			# Strip any IP:port pairs in message to only IP. Port usually
+			# changes with each error message
+			ip, port = RE_IP_PORT.search(message).groups()
+			message = message.replace("%s:%s" % (ip, port), ip)
 		if message in self.error_messages:
 			# Same error is already displayed
 			log.info("(repeated) %s", message)
@@ -795,7 +820,7 @@ class App(Gtk.Application, TimerManager):
 		if message in self.error_messages:
 			self.error_messages.remove(message)
 	
-	def cb_syncthing_folder_rejected(self, daemon, nid, rid):
+	def cb_syncthing_folder_rejected(self, daemon, nid, rid, label):
 		if (nid, rid) in self.error_messages:
 			# Store as error message and don't display twice
 			return
@@ -806,13 +831,13 @@ class App(Gtk.Application, TimerManager):
 			can_fix = True
 		markup = _('%(device)s wants to share folder "%(folder)s". Add new folder?') % {
 			'device' : "<b>%s</b>" % device,
-			'folder' : "<b>%s</b>" % rid
+			'folder' : "<b>%s</b>" % (label or rid)
 			}
 		r = RIBar("", Gtk.MessageType.WARNING,)
 		r.get_label().set_markup(markup)
 		if can_fix:
 			r.add_button(RIBar.build_button(_("_Add")), RESPONSE_FIX_FOLDER_ID)
-		self.show_error_box(r, {"nid" : nid, "rid" : rid} )
+		self.show_error_box(r, {"nid" : nid, "rid" : rid, "label" : label } )
 	
 	def cb_syncthing_device_rejected(self, daemon, nid, name, address):
 		# Remove port from address, it's random by default anyway
@@ -835,7 +860,7 @@ class App(Gtk.Application, TimerManager):
 		r.get_label().set_markup(markup)
 		r.add_button(RIBar.build_button(_("_Add")), RESPONSE_FIX_NEW_DEVICE)
 		r.add_button(RIBar.build_button(_("_Ignore")), RESPONSE_FIX_IGNORE)
-		self.show_error_box(r, {"nid" : nid, "address" : address} )
+		self.show_error_box(r, {"nid" : nid, "name" : name, "address" : address} )
 	
 	def cb_syncthing_my_id_changed(self, daemon, device_id):
 		if device_id in self.devices:
@@ -873,21 +898,16 @@ class App(Gtk.Application, TimerManager):
 			# Check for new version, if enabled
 			self.check_for_upgrade()
 	
-	def cb_syncthing_system_data(self, daemon, mem, cpu, announce):
+	def cb_syncthing_system_data(self, daemon, mem, cpu, d_failed, d_total):
 		if self.daemon.get_my_id() in self.devices:
 			# Update my device display
 			device = self.devices[self.daemon.get_my_id()]
 			device["ram"] = sizeof_fmt(mem)
 			device["cpu"] = "%3.2f%%" % (cpu)
-			if announce is None or len(announce) == 0:
+			if d_total == 0:
 				device["announce"] = _("disabled")
-			elif len(announce) == 1:
-				device["announce"] = _("Online") if announce.values()[0] else _("offline")
-			else:
-				device["announce"] = _("%(online)s/%(total)s online") % {
-						'online' : len([ x for x in announce.values() if x ]),
-						'total'  : len(announce)
-					}
+			else:			
+				device["announce"] = "%s/%s" % (d_total - d_failed, d_total)
 	
 	def cb_syncthing_device_added(self, daemon, nid, name, used, data):
 		self.show_device(nid, name,
@@ -1052,17 +1072,17 @@ class App(Gtk.Application, TimerManager):
 				self.animate_status()
 			elif self.any_device_online():
 				# Daemon is online and idle
-				self.statusicon.set("si-idle", _("Up to Date"))
+				self.statusicon.set("si-%s-idle" % (self.config['icon_theme'],), _("Up to Date"))
 				self["menu-si-status"].set_label(_("Up to Date"))
 				self.cancel_timer("icon")
 			else:
 				# Daemon is online, but there is no remote device connected
-				self.statusicon.set("si-unknown", _("All devices offline"))
+				self.statusicon.set("si-%s-unknown" % (self.config['icon_theme'],), _("All devices offline"))
 				self["menu-si-status"].set_label(_("All devices offline"))
 				self.cancel_timer("icon")
 		else:
 			# Still connecting to syncthing daemon
-			self.statusicon.set("si-unknown", _("Connecting to Syncthing daemon..."))
+			self.statusicon.set("si-%s-unknown" % (self.config['icon_theme'],), _("Connecting to Syncthing daemon..."))
 			self["menu-si-status"].set_label(_("Connecting to Syncthing daemon..."))
 			self.cancel_timer("icon")
 	
@@ -1071,7 +1091,7 @@ class App(Gtk.Application, TimerManager):
 		if self.timer_active("icon"):
 			# Already animating
 			return
-		self.statusicon.set("si-syncing-%s" % (self.sync_animation,))
+		self.statusicon.set("si-%s-%s" % (self.config['icon_theme'], self.sync_animation,))
 		self.sync_animation += 1
 		if self.sync_animation >= SI_FRAMES:
 			self.sync_animation = 0
@@ -1577,6 +1597,7 @@ class App(Gtk.Application, TimerManager):
 				self.daemon.handler_disconnect(handler_id)
 			self.show()
 			e = FolderEditorDialog(self, True, None, path)
+			e.call_after_loaded(e.fill_folder_id, generate_folder_id(), False)
 			e.load()
 			e.show(self["window"])
 		
@@ -1754,8 +1775,9 @@ class App(Gtk.Application, TimerManager):
 		path = os.path.expanduser(box["path"])
 		if IS_WINDOWS:
 			# Don't attempt anything, use Windows Explorer on Windows
+			windows = os.environ("SystemRoot")
 			path = path.replace("/", "\\")
-			os.system('explorer "%s"' % (path,))
+			os.system('%s\\explorer.exe "%s"' % (windows, path))
 		else:
 			# Try to use any of following, known commands to
 			# display directory contents
@@ -1925,10 +1947,14 @@ class App(Gtk.Application, TimerManager):
 					e = FolderEditorDialog(self, True, additional_data["rid"])
 					e.call_after_loaded(e.mark_device, additional_data["nid"])
 					e.call_after_loaded(e.fill_folder_id, additional_data["rid"])
+					if additional_data["label"]:
+						e.call_after_loaded(e["vlabel"].set_text, additional_data["label"])
 					e.load()
 					e.show(self["window"])
 		elif response_id == RESPONSE_FIX_NEW_DEVICE:
 			e = DeviceEditorDialog(self, True, additional_data["nid"])
+			if additional_data["name"]:
+				e.call_after_loaded(e["vname"].set_text, additional_data["name"])
 			e.load()
 			e.show(self["window"])
 		elif response_id == RESPONSE_FIX_IGNORE:
